package pathgame.gameplay;

import pathgame.tilemap.TileMap;

public class Player
{
    private static final float SPEED = 0.125f;

    private float lastX = 0;
    private float lastY = 0;
    private float x = 0;
    private float y = 0;

    private float velX = 0;
    private float velY = 0;
    private boolean isMoving = false;
    private int currSpeedSlowdown = 1;

    private final int energySupply;
    private int energyUsed = 0;
    private final int objectivesAmount;
    private int objectivesVisited = 0;
    private lastPlayerSteps lastSteps = new lastPlayerSteps();
    //private int minusEnergy = 0;

    public Player(int energySupply, int objectivesAmount)
    {
        this.energySupply = energySupply;
        this.objectivesAmount = objectivesAmount; 
    }

    public float getLastX()
    {
        return lastX;
    }

    public float getLastY()
    {
        return lastY;
    }

    public float getX()
    {
        return x;
    }

    public float getY()
    {
        return y;
    }

    public void tick(TileMap map)
    {
        
        currSpeedSlowdown = map.getTile(Math.round(x), Math.round(y)).getEnergyCost();
        lastX = x;
        lastY = y;

        if(velY != 0)
        {
            //System.out.println(velY + " " + currSpeedSlowdown + " " + y);
        }

        if(isOnTile())
        {
            
            velX = 0.0f;
            velY = 0.0f;
            if(isMoving)
            {
                ++objectivesVisited;//TODO check for objective
                //System.out.print(map.getTile((int)x,(int) y).getEnergyCost());
                lastSteps.addMinus(currSpeedSlowdown, x, y);
                //minusEnergy = currSpeedSlowdown;
                energyUsed += currSpeedSlowdown;
                //currSpeedSlowdown = 1;
                //TODO: +Energy
                //Tile currTile = map.getTile((int)x,(int) y);

            }
            isMoving = false;
        }

        if(Keys.LEFT_KEY.isDown() && !isMoving && x > 0)
        {
            velX = -SPEED;
            isMoving = true;
        }
        else if(Keys.RIGHT_KEY.isDown() && !isMoving && x < map.getWidth() - 1)
        {
            velX = SPEED;
            isMoving = true;
        }
        else if(Keys.UP_KEY.isDown() && !isMoving && y > 0)
        {
            velY = -SPEED;
            isMoving = true;
        }
        else if(Keys.DOWN_KEY.isDown() && !isMoving && y < map.getHeight() - 1)
        {
            velY = SPEED;
            isMoving = true;
        }

        float moveX = Math.abs(velX / currSpeedSlowdown);
        float moveY = Math.abs(velY / currSpeedSlowdown);
        if(velX < 0.0f)
        {
            float d = x - (float) Math.floor(x);
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveX = -Math.min(moveX, d);
        }
        else if(velX > 0.0f)
        {
            float d = (float) Math.ceil(x) - x;
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveX = Math.min(moveX, d);
        }
        else if(velY < 0.0f)
        {
            float d = y - (float) Math.floor(y);
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveY = -Math.min(moveY, d);
        }
        else if(velY > 0.0f)
        {
            float d = (float) Math.ceil(y) - y;
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveY = Math.min(moveY, d);
        }

        System.out.println(moveX);
        x += moveX;
        y += moveY;
    }

    private boolean isOnTile()
    {
        return Math.abs(x - Math.round(x)) < 0.01f && Math.abs(y - Math.round(y)) < 0.01f;
    }

    public int getEnergySupply()
    {
        return energySupply;
    }

    public int getEnergyLeft()
    {
        return energySupply - energyUsed;
    }

    public int getEnergyUsed()
    {
        return energyUsed;
    }
    
    public int getObjectivesAmount()
    {
        return objectivesAmount;
    }
    
    public int getObjectivesVisited()
    {
        return objectivesVisited;
    }
    
    public lastPlayerSteps getLastSteps()
    {
        return lastSteps;
    }
//    public int getMinusEnergy()
//    {
//        int mEnergy = minusEnergy;
//        minusEnergy = 0;
//        return mEnergy;
//    }
}
